@@ -1,32 +1,40 @@
 # Brief help module to define the exception we use for early exits.
-package Zonemaster::Exception::NormalExit;
+package Zonemaster::Engine::Exception::NormalExit;
 use Moose;
-extends 'Zonemaster::Exception';
+extends 'Zonemaster::Engine::Exception';
 
 # The actual interesting module.
-package Zonemaster::CLI v1.0.5;
+package Zonemaster::CLI;
 
 use 5.014002;
+
+use strict;
 use warnings;
 
+use version; our $VERSION = version->declare( "v5.0.2" );
+
 use Locale::TextDomain 'Zonemaster-CLI';
 use Moose;
-with 'MooseX::Getopt';
+with 'MooseX::Getopt::GLD' => { getopt_conf => [ 'pass_through' ] };
 
-use Zonemaster;
-use Zonemaster::Logger::Entry;
-use Zonemaster::Translator;
-use Zonemaster::Util qw[pod_extract_for];
-use Zonemaster::Exception;
-use Scalar::Util qw[blessed];
 use Encode;
-use Net::LDNS;
-use POSIX qw[setlocale LC_MESSAGES];
-use List::Util qw[max];
-use Text::Reflow qw[reflow_string];
+use File::Slurp;
 use JSON::XS;
+use List::Util qw[max];
+use POSIX qw[setlocale LC_MESSAGES LC_CTYPE];
+use Scalar::Util qw[blessed];
+use Socket qw[AF_INET AF_INET6];
+use Text::Reflow qw[reflow_string];
+use Try::Tiny;
+use Zonemaster::Engine;
+use Zonemaster::Engine::Exception;
+use Zonemaster::Engine::Logger::Entry;
+use Zonemaster::Engine::Translator;
+use Zonemaster::Engine::Util qw[parse_hints pod_extract_for];
+use Zonemaster::Engine::Zone;
+use Zonemaster::LDNS;
 
-our %numeric = Zonemaster::Logger::Entry->levels;
+our %numeric = Zonemaster::Engine::Logger::Entry->levels;
 our $JSON    = JSON::XS->new->allow_blessed->convert_blessed->canonical;
 
 STDOUT->autoflush( 1 );
@@ -40,10 +48,14 @@ has 'version' => (
 );
 
 has 'level' => (
-    is       => 'ro',
-    isa      => 'Str',
-    required => 0,
-    default  => 'NOTICE',
+    is            => 'ro',
+    isa           => 'Str',
+    required      => 0,
+    default       => 'NOTICE',
+    initializer   => sub {
+        my ( $self, $value, $set, $attr ) = @_;
+        $set->( uc $value );
+    },
     documentation =>
       __( 'The minimum severity level to display. Must be one of CRITICAL, ERROR, WARNING, NOTICE, INFO or DEBUG.' ),
 );
@@ -63,16 +75,22 @@ has 'json' => (
 );
 
 has 'json_stream' => (
+    traits        => [ 'Getopt' ],
     is            => 'ro',
     isa           => 'Bool',
     default       => 0,
+    cmd_aliases   => 'json_stream',
+    cmd_flag      => 'json-stream',
     documentation => __( 'Flag indicating if output should be streaming JSON or not.' ),
 );
 
 has 'json_translate' => (
+    traits        => [ 'Getopt' ],
     is            => 'ro',
     isa           => 'Bool',
     default       => 0,
+    cmd_aliases   => 'json_translate',
+    cmd_flag      => 'json-translate',
     documentation => __( 'Flag indicating if streaming JSON output should include the translated message of the tag or not.' ),
 );
 
@@ -91,25 +109,48 @@ has 'time' => (
 );
 
 has 'show_level' => (
+    traits        => [ 'Getopt' ],
     is            => 'ro',
     isa           => 'Bool',
+    cmd_aliases   => 'show_level',
+    cmd_flag      => 'show-level',
     documentation => __( 'Print level on entries.' ),
     default       => 1,
 );
 
 has 'show_module' => (
+    traits        => [ 'Getopt' ],
     is            => 'ro',
     isa           => 'Bool',
+    cmd_aliases   => 'show_module',
+    cmd_flag      => 'show-module',
     documentation => __( 'Print the name of the module on entries.' ),
     default       => 0,
 );
 
+has 'show_testcase' => (
+    traits        => [ 'Getopt' ],
+    is            => 'ro',
+    isa           => 'Bool',
+    cmd_aliases   => 'show_testcase',
+    cmd_flag      => 'show-testcase',
+    documentation => __( 'Print the name of the test case on entries.' ),
+    default       => 0,
+);
+
 has 'ns' => (
     is            => 'ro',
     isa           => 'ArrayRef',
     documentation => __( 'A name/ip string giving a nameserver for undelegated tests, or just a name which will be looked up for IP addresses. Can be given multiple times.' ),
 );
 
+has 'hints' => (
+    is            => 'ro',
+    isa           => 'Str',
+    required      => 0,
+    documentation => __( 'Name of a root hints file to override the defaults.' ),
+);
+
 has 'save' => (
     is            => 'ro',
     isa           => 'Str',
@@ -125,25 +166,26 @@ has 'restore' => (
 );
 
 has 'ipv4' => (
-    is      => 'ro',
-    isa     => 'Bool',
-    default => 1,
+    is            => 'ro',
+    isa           => 'Bool',
     documentation =>
       __( 'Flag to permit or deny queries being sent via IPv4. --ipv4 permits IPv4 traffic, --no-ipv4 forbids it.' ),
 );
 
 has 'ipv6' => (
-    is      => 'ro',
-    isa     => 'Bool',
-    default => 1,
+    is            => 'ro',
+    isa           => 'Bool',
     documentation =>
       __( 'Flag to permit or deny queries being sent via IPv6. --ipv6 permits IPv6 traffic, --no-ipv6 forbids it.' ),
 );
 
 has 'list_tests' => (
+    traits        => [ 'Getopt' ],
     is            => 'ro',
     isa           => 'Bool',
     default       => 0,
+    cmd_aliases   => 'list_tests',
+    cmd_flag      => 'list-tests',
     documentation => __( 'Instead of running a test, list all available tests.' ),
 );
 
@@ -157,26 +199,26 @@ has 'test' => (
 );
 
 has 'stop_level' => (
+    traits        => [ 'Getopt' ],
     is            => 'ro',
     isa           => 'Str',
     required      => 0,
+    initializer   => sub {
+        my ( $self, $value, $set, $attr ) = @_;
+        $set->( uc $value );
+    },
+    cmd_aliases   => 'stop_level',
+    cmd_flag      => 'stop-level',
     documentation => __(
 'As soon as a message at this level or higher is logged, execution will stop. Must be one of CRITICAL, ERROR, WARNING, NOTICE, INFO or DEBUG.'
     )
 );
 
-has 'config' => (
+has 'profile' => (
     is            => 'ro',
     isa           => 'Str',
     required      => 0,
-    documentation => __( 'Name of configuration file to load.' ),
-);
-
-has 'policy' => (
-    is            => 'ro',
-    isa           => 'Str',
-    required      => 0,
-    documentation => __( 'Name of policy file to load.' ),
+    documentation => __( 'Name of profile file to load. (DEFAULT)' ),
 );
 
 has 'ds' => (
@@ -197,13 +239,13 @@ has 'progress' => (
     is            => 'ro',
     isa           => 'Bool',
     default       => !!( -t STDOUT ),
-    documentation => __( 'Boolean flag for activity indicator. Defaults to on if STDOUT is a tty, off if it is not.' ),
+    documentation => __( 'Boolean flag for activity indicator. Defaults to on if STDOUT is a tty, off if it is not. Disable with --no-progress.' ),
 );
 
 has 'encoding' => (
-    is      => 'ro',
-    isa     => 'Str',
-    default => sub {
+    is            => 'ro',
+    isa           => 'Str',
+    default       => sub {
         my $locale = $ENV{LC_CTYPE} // 'C';
         my ( $e ) = $locale =~ m|\.(.*)$|;
         $e //= 'UTF-8';
@@ -220,35 +262,32 @@ has 'nstimes' => (
     documentation => __('At the end of a run, print a summary of the times the zone\'s name servers took to answer.'),
 );
 
-has 'dump_config' => (
-    is => 'ro',
-    isa => 'Bool',
-    required => 0,
-    default => 0,
-    documentation => __( 'Print the effective configuration used in JSON format, then exit.' ),
-);
-
-has 'dump_policy' => (
-    is => 'ro',
-    isa => 'Bool',
-    required => 0,
-    default => 0,
-    documentation => __( 'Print the effective policy used in JSON format, then exit.' ),
+has 'dump_profile' => (
+    traits        => [ 'Getopt' ],
+    is            => 'ro',
+    isa           => 'Bool',
+    required      => 0,
+    default       => 0,
+    cmd_aliases   => 'dump_profile',
+    cmd_flag      => 'dump-profile',
+    documentation => __( 'Print the effective profile used in JSON format, then exit.' ),
 );
 
 has 'sourceaddr' => (
-    is => 'ro',
-    isa => 'Maybe[Str]',
-    required => 0,
-    default => undef,
-    documentation => __( 'Local IP address that the test engine should try to send its requests from.' ),
+    is            => 'ro',
+    isa           => 'Str',
+    required      => 0,
+    documentation => __(
+            'Source IP address used to send queries. '
+          . 'Setting an IP address not correctly configured on a local network interface causes cryptic error messages.'
+    ),
 );
 
 has 'elapsed' => (
-    is => 'ro',
-    isa => 'Bool',
-    required => 0,
-    default => 0,
+    is            => 'ro',
+    isa           => 'Bool',
+    required      => 0,
+    default       => 0,
     documentation => 'Print elapsed time at end of run.',
 );
 
@@ -258,12 +297,25 @@ sub run {
     my %counter;
     my $printed_something;
 
+    if ( grep /^-/, @{ $self->extra_argv } ) {
+        print "Unknown option: ", join( q{ }, grep /^-/, @{ $self->extra_argv } ), "\n";
+        print "Run \"zonemaster-cli -h\" to get the valid options\n";
+        exit;
+    }
+
     if ( $self->locale ) {
-        my $loc = setlocale( LC_MESSAGES, $self->locale );
-        if ( not defined $loc ) {
-            printf STDERR __( "Warning: setting locale %s failed (is it installed on this system?).\n\n" ),
-              $self->locale;
-        }
+        undef $ENV{LANGUAGE};
+        $ENV{LC_ALL} = $self->locale;
+    }
+
+    # Set LC_MESSAGES and LC_CTYPE separately (https://www.gnu.org/software/gettext/manual/html_node/Triggering.html#Triggering)
+    if ( not defined setlocale( LC_MESSAGES, "" ) ) {
+        printf STDERR __( "Warning: setting locale category LC_MESSAGES to %s failed (is it installed on this system?).\n\n" ),
+        $ENV{LANGUAGE} || $ENV{LC_ALL} || $ENV{LC_MESSAGES};
+    }
+    if ( not defined setlocale( LC_CTYPE, "" ) ) {
+        printf STDERR __( "Warning: setting locale category LC_CTYPE to %s failed (is it installed on this system?).\n\n" ),
+        $ENV{LC_ALL} || $ENV{LC_CTYPE};
     }
 
     if ( $self->version ) {
@@ -275,32 +327,37 @@ sub run {
         print_test_list();
     }
 
-    Zonemaster->config->ipv4_ok(0+$self->ipv4);
-    Zonemaster->config->ipv6_ok(0+$self->ipv6);
-
     if ($self->sourceaddr) {
-        Zonemaster->config->resolver_source($self->sourceaddr);
+
+        Zonemaster::Engine::Profile->effective->set( q{resolver.source}, $self->sourceaddr );
     }
 
-    if ( $self->policy ) {
-        say __( "Loading policy from " ) . $self->policy . '.' if not ($self->dump_config or $self->dump_policy);
-        Zonemaster->config->load_policy_file( $self->policy );
+    # Filehandle for diagnostics output
+    my $fh_diag = ( $self->json or $self->json_stream or $self->raw )
+      ? *STDERR     # Structured output mode (e.g. JSON)
+      : *STDOUT;    # Human readable output mode
+
+    if ( $self->profile ) {
+        say $fh_diag __x( "Loading profile from {path}.", path => $self->profile );
+        my $json    = read_file( $self->profile );
+        my $foo     = Zonemaster::Engine::Profile->from_json( $json );
+        my $profile = Zonemaster::Engine::Profile->default;
+        $profile->merge( $foo );
+        Zonemaster::Engine::Profile->effective->merge( $profile );
     }
 
-    if ( $self->config ) {
-        say __( "Loading configuration from " ) . $self->config . '.' if not ($self->dump_config or $self->dump_policy);
-        Zonemaster->config->load_config_file( $self->config );
+    # These two must come after any profile from command line has been loaded
+    # to make any IPv4/IPv6 option override the profile setting.
+    if ( defined ($self->ipv4) ) {
+        Zonemaster::Engine::Profile->effective->set( q{net.ipv4}, 0+$self->ipv4 );
+    }
+    if ( defined ($self->ipv6) ) {
+        Zonemaster::Engine::Profile->effective->set( q{net.ipv6}, 0+$self->ipv6 );
     }
 
-    if ( $self->dump_config ) {
-        do_dump_config();
-    }
 
-    if ( $self->dump_policy ) {
-        foreach my $mod (Zonemaster->modules) {
-            Zonemaster->config->load_module_policy($mod)
-        }
-        do_dump_policy();
+    if ( $self->dump_profile ) {
+        do_dump_profile();
     }
 
     if ( $self->stop_level and not defined( $numeric{ $self->stop_level } ) ) {
@@ -312,53 +369,70 @@ sub run {
     }
 
     my $translator;
-    $translator = Zonemaster::Translator->new unless ( $self->raw or $self->json or $self->json_stream );
+    $translator = Zonemaster::Engine::Translator->new unless ( $self->raw or $self->json or $self->json_stream );
     $translator->locale( $self->locale ) if $translator and $self->locale;
-    eval { $translator->data } if $translator;    # Provoke lazy loading of translation data
 
     my $json_translator;
     if ( $self->json_translate ) {
-        $json_translator = Zonemaster::Translator->new;
+        $json_translator = Zonemaster::Engine::Translator->new;
         $json_translator->locale( $self->locale ) if $self->locale;
-        eval { $json_translator->data };
     }
 
     if ( $self->restore ) {
-        Zonemaster->preload_cache( $self->restore );
+        Zonemaster::Engine->preload_cache( $self->restore );
     }
 
     # Callback defined here so it closes over the setup above.
-    Zonemaster->logger->callback(
+    Zonemaster::Engine->logger->callback(
         sub {
             my ( $entry ) = @_;
 
-            $self->print_spinner() unless $self->json_stream;
+            $self->print_spinner() if $fh_diag eq *STDOUT;
 
             $counter{ uc $entry->level } += 1;
 
-            if ( $numeric{ uc $entry->level } >= $numeric{ uc $self->level } ) {
+            if ( $numeric{ uc $entry->level } >= $numeric{ $self->level } ) {
                 $printed_something = 1;
 
                 if ( $translator ) {
+                    my $header = q{};
                     if ( $self->time ) {
-                        printf "%7.2f ", $entry->timestamp;
+                        $header .= sprintf "%7.2f ", $entry->timestamp;
                     }
 
                     if ( $self->show_level ) {
-                        printf "%-9s ", __( $entry->level );
+                        $header .= sprintf "%-9s ", translate_severity( $entry->level );
                     }
 
                     if ( $self->show_module ) {
-                        printf "%-12s ", $entry->module;
+                        $header .= sprintf "%-12s ", $entry->module;
                     }
 
-                    say $translator->translate_tag( $entry );
+                    if ( $self->show_testcase ) {
+                        $header .= sprintf "%-14s ", $entry->testcase;
+                    }
+
+                    print $header;
+
+                    if ( $entry->level eq q{DEBUG3} and scalar( keys %{$entry->args} ) == 1 and defined $entry->args->{packet} ) {
+                        my $packet = $entry->args->{packet};
+                        my $padding = q{ } x length $header;
+                        $entry->args->{packet} = q{};
+                        say $translator->translate_tag( $entry );
+                        foreach my $line ( split /\n/, $packet ) {
+                            print $padding, $line, "\n";
+                        }
+                    }
+                    else {
+                        say $translator->translate_tag( $entry );
+                    }
                 }
                 elsif ( $self->json_stream ) {
                     my %r;
 
                     $r{timestamp} = $entry->timestamp;
                     $r{module}    = $entry->module;
+                    $r{testcase}  = $entry->testcase;
                     $r{tag}       = $entry->tag;
                     $r{level}     = $entry->level;
                     $r{args}      = $entry->args if $entry->args;
@@ -369,21 +443,39 @@ sub run {
                 elsif ( $self->json ) {
                     # Don't do anything
                 }
-                elsif ( $self->show_module ) {
-                    printf "%7.2f %-9s %-12s %s\n", $entry->timestamp, $entry->level, $entry->module, $entry->string;
-                }
                 else {
-                    printf "%7.2f %-9s %s\n", $entry->timestamp, $entry->level, $entry->string;
+                    my $prefix = sprintf "%7.2f %-9s ", $entry->timestamp, $entry->level;
+                    if ( $self->show_module ) {
+                        $prefix .= sprintf "%-12s ", $entry->module;
+                    }
+                    if ( $self->show_testcase ) {
+                        $prefix .= sprintf "%-14s ", $entry->testcase;
+                    }
+                    $prefix .= $entry->tag;
+
+                    my $message = $entry->string;
+                    $message =~ s/^[A-Z0-9:_]+//;    # strip MODULE:TAG, they're coming in $prefix instead
+                    my @lines = split /\n/, $message;
+
+                    printf "%s%s %s\n", $prefix, ' ', shift @lines;
+                    for my $line ( @lines ) {
+                        printf "%s%s %s\n", $prefix, '>', $line;
+                    }
                 }
             } ## end if ( $numeric{ uc $entry...})
-            if ( $self->stop_level and $numeric{ uc $entry->level } >= $numeric{ uc $self->stop_level } ) {
-                die( Zonemaster::Exception::NormalExit->new( { message => "Saw message at level " . $entry->level } ) );
+            if ( $self->stop_level and $numeric{ uc $entry->level } >= $numeric{ $self->stop_level } ) {
+                die( Zonemaster::Engine::Exception::NormalExit->new( { message => "Saw message at level " . $entry->level } ) );
             }
         }
     );
 
-    if ( $self->config or $self->policy ) {
-        print "\n";    # Cosmetic
+    if ( $self->profile ) {
+        # Separate initialization from main output in human readable output mode
+        print "\n" if $fh_diag eq *STDOUT;
+    }
+
+    if ( scalar @{ $self->extra_argv } > 1 ) {
+        die __( "Only one domain can be given for testing. Did you forget to prepend an option with '--<OPTION>'?\n" );
     }
 
     my ( $domain ) = @{ $self->extra_argv };
@@ -391,6 +483,34 @@ sub run {
         die __( "Must give the name of a domain to test.\n" );
     }
 
+    if ( $domain =~ m/\.\./i ) {
+        die __( "The domain name contains consecutive dots.\n" );
+    }
+
+    $domain =~ s/\.$// unless $domain eq '.';
+    $domain = $self->to_idn( $domain );
+
+    if ( defined $self->hints ) {
+        my $hints_data;
+        try {
+            my $hints_text = read_file( $self->hints );
+            $hints_data = parse_hints( $hints_text )
+        }
+        catch {
+            die "Error loading hints file: $_";
+        }
+        Zonemaster::Engine::Recursor->remove_fake_addresses( '.' );
+        Zonemaster::Engine::Recursor->add_fake_addresses( '.', $hints_data );
+    }
+
+    if ( $self->ns and @{ $self->ns } > 0 ) {
+        $self->add_fake_delegation( $domain );
+    }
+
+    if ( $self->ds and @{ $self->ds } ) {
+        $self->add_fake_ds( $domain );
+    }
+
     if ( $translator ) {
         if ( $self->time ) {
             print __( 'Seconds ' );
@@ -401,6 +521,9 @@ sub run {
         if ( $self->show_module ) {
             print __( 'Module       ' );
         }
+        if ( $self->show_testcase ) {
+            print __( 'Testcase       ' );
+        }
         say __( 'Message' );
 
         if ( $self->time ) {
@@ -412,34 +535,27 @@ sub run {
         if ( $self->show_module ) {
             print __( '============ ' );
         }
+        if ( $self->show_testcase ) {
+            print __( '============== ' );
+        }
         say __( '=======' );
     } ## end if ( $translator )
 
-    $domain = $self->to_idn( $domain );
-
-    if ( $self->ns and @{ $self->ns } > 0 ) {
-        $self->add_fake_delegation( $domain );
-    }
-
-    if ( $self->ds and @{ $self->ds } ) {
-        $self->add_fake_ds( $domain );
-    }
-
     # Actually run tests!
     eval {
         if ( $self->test and @{ $self->test } > 0 ) {
             foreach my $t ( @{ $self->test } ) {
                 my ( $module, $method ) = split( '/', $t, 2 );
                 if ( $method ) {
-                    Zonemaster->test_method( $module, $method, Zonemaster->zone( $domain ) );
+                    Zonemaster::Engine->test_method( $module, $method, Zonemaster::Engine->zone( $domain ) );
                 }
                 else {
-                    Zonemaster->test_module( $module, $domain );
+                    Zonemaster::Engine->test_module( $module, $domain );
                 }
             }
         }
         else {
-            Zonemaster->test_zone( $domain );
+            Zonemaster::Engine->test_zone( $domain );
         }
     };
     if ( $translator ) {
@@ -450,7 +566,7 @@ sub run {
 
     if ( $@ ) {
         my $err = $@;
-        if ( blessed $err and $err->isa( "Zonemaster::Exception::NormalExit" ) ) {
+        if ( blessed $err and $err->isa( "Zonemaster::Engine::Exception::NormalExit" ) ) {
             say STDERR "Exited early: " . $err->message;
         }
         else {
@@ -462,13 +578,13 @@ sub run {
         say __( "\n\n   Level\tNumber of log entries" );
         say "   =====\t=====================";
         foreach my $level ( sort { $numeric{$b} <=> $numeric{$a} } keys %counter ) {
-            printf __( "%8s\t%5d entries.\n" ), __( $level ), $counter{$level};
+            printf __( "%8s\t%5d entries.\n" ), translate_severity( $level ), $counter{$level};
         }
     }
 
     if ( $self->nstimes ) {
-        my $zone = Zonemaster->zone( $domain );
-        my $max = max map { length( "$_" ) } @{ $zone->ns };
+        my $zone = Zonemaster::Engine->zone( $domain );
+        my $max = max map { length( "$_" ) } ( @{ $zone->ns }, q{Server} );
 
         print "\n";
         printf "%${max}s %s\n", 'Server', ' Max (ms)      Min      Avg   Stddev   Median     Total';
@@ -486,16 +602,16 @@ sub run {
     }
 
     if ($self->elapsed) {
-        my $last = Zonemaster->logger->entries->[-1];
+        my $last = Zonemaster::Engine->logger->entries->[-1];
         printf "Total test run time: %0.1f seconds.\n", $last->timestamp;
     }
 
     if ( $self->json ) {
-        say Zonemaster->logger->json( $self->level );
+        say Zonemaster::Engine->logger->json( $self->level );
     }
 
     if ( $self->save ) {
-        Zonemaster->save_cache( $self->save );
+        Zonemaster::Engine->save_cache( $self->save );
     }
 
     return;
@@ -503,29 +619,39 @@ sub run {
 
 sub add_fake_delegation {
     my ( $self, $domain ) = @_;
+    my @ns_with_no_ip;
     my %data;
 
     foreach my $pair ( @{ $self->ns } ) {
         my ( $name, $ip ) = split( '/', $pair, 2 );
 
         if ( not $name ) {
-            say STDERR "--ns must have be a name or a name/ip pair.";
+            say STDERR __( "--ns must be a name or a name/ip pair." );
             exit( 1 );
         }
 
+        if ( $name =~ m/\.\./i ) {
+            say STDERR __x( "The name of the nameserver '{nsname}' contains consecutive dots.", nsname => $name );
+            exit ( 1 );
+        }
+
+        $name =~ s/\.$// unless $name eq '.';
+
         if ($ip) {
             push @{ $data{ $self->to_idn( $name ) } }, $ip;
         }
         else {
-            my $n = $self->to_idn( $name );
-            my @ips = Net::LDNS->new->name2addr($n);
-            push @{ $data{$n} }, $_ for @ips;
+            push @ns_with_no_ip, $self->to_idn($name);
+        }
+    }
+    foreach my $ns ( @ns_with_no_ip ) {
+        if ( not exists $data{ $ns } ) {
+            $data{ $ns } = undef;
         }
     }
 
-    Zonemaster->add_fake_delegation( $domain => \%data );
+    return Zonemaster::Engine->add_fake_delegation( $domain => \%data );
 
-    return;
 }
 
 sub add_fake_ds {
@@ -537,19 +663,19 @@ sub add_fake_ds {
         push @data, { keytag => $tag, algorithm => $algo, type => $type, digest => $digest };
     }
 
-    Zonemaster->add_fake_ds( $domain => \@data );
+    Zonemaster::Engine->add_fake_ds( $domain => \@data );
 
     return;
 }
 
 sub print_versions {
     say 'CLI version:    ' . __PACKAGE__->VERSION;
-    say 'Engine version: ' . $Zonemaster::VERSION;
+    say 'Engine version: ' . $Zonemaster::Engine::VERSION;
     say "\nTest module versions:";
 
-    my %methods = Zonemaster->all_methods;
+    my %methods = Zonemaster::Engine->all_methods;
     foreach my $module ( sort keys %methods ) {
-        my $mod = "Zonemaster::Test::$module";
+        my $mod = "Zonemaster::Engine::Test::$module";
         say "\t$module: " . $mod->version;
     }
 
@@ -575,17 +701,17 @@ sub to_idn {
         return $str;
     }
 
-    if ( Net::LDNS::has_idn() ) {
-        return Net::LDNS::to_idn( decode( $self->encoding, $str ) );
+    if ( Zonemaster::LDNS::has_idn() ) {
+        return Zonemaster::LDNS::to_idn( decode( $self->encoding, $str ) );
     }
     else {
-        say STDERR __( "Warning: Net::LDNS not compiled with libidn, cannot handle non-ASCII names correctly." );
+        say STDERR __( "Warning: Zonemaster::LDNS not compiled with IDN support, cannot handle non-ASCII names correctly." );
         return $str;
     }
 }
 
 sub print_test_list {
-    my %methods = Zonemaster->all_methods;
+    my %methods = Zonemaster::Engine->all_methods;
     my $maxlen  = max map {
         map { length( $_ ) }
           @$_
@@ -612,16 +738,37 @@ sub print_test_list {
     exit( 0 );
 } ## end sub print_test_list
 
-sub do_dump_policy {
+sub do_dump_profile {
     my $json = JSON::XS->new->canonical->pretty;
-    print $json->encode(Zonemaster->config->policy);
+
+    print $json->encode( Zonemaster::Engine::Profile->effective->{ q{profile} } );
+
     exit;
 }
 
-sub do_dump_config {
-    my $json = JSON::XS->new->canonical->pretty;
-    print $json->encode(Zonemaster->config->get);
-    exit;
+sub translate_severity {
+    my $severity = shift;
+    if ( $severity eq "DEBUG" ) {
+        return __( "DEBUG" );
+    }
+    elsif ( $severity eq "INFO" ) {
+        return __( "INFO" );
+    }
+    elsif ( $severity eq "NOTICE" ) {
+        return __( "NOTICE" );
+    }
+    elsif ( $severity eq "WARNING" ) {
+        return __( "WARNING" );
+    }
+    elsif ( $severity eq "ERROR" ) {
+        return __( "ERROR" );
+    }
+    elsif ( $severity eq "CRITICAL" ) {
+        return __( "CRITICAL" );
+    }
+    else {
+        return $severity;
+    }
 }
 
 1;
@@ -634,3 +781,18 @@ __END__
 =head1 NAME
 
 Zonemaster::CLI - run Zonemaster tests from the command line
+
+=head1 AUTHORS
+
+Vincent Levigneron <vincent.levigneron at nic.fr>
+- Current maintainer
+
+Calle Dybedahl <calle at init.se>
+- Original author
+
+=head1 LICENSE
+
+This is free software under a 2-clause BSD license. The full text of the license can
+be found in the F<LICENSE> file included with this distribution.
+
+=cut
